#!/usr/bin/env python

##############################################################
#                     (C) SANDULEAC DAN                      #
#                           EVALUATOR                        #
#                  <<  Console interface  >>                 #
#                 RELEASED UNDER GPL LICENCE                 #
##############################################################

import evalcore
import sys as _sys, os as _os, re as _re
from os.path import join, isfile, isdir

# Don't modify anything below
# To configure test db directory, use config.py
# ------------------------------------------------------------
# some constants defined here

__ESC = chr(27)
__color = {
    'green' : __ESC+'[1;32m',
    'green_i' : __ESC+'[0;32m',
    'red'   : __ESC+'[1;31m',
    'else'  : __ESC+'[1;35m',
    'maialb': __ESC+'[1;37m',
    'normal': __ESC+'[m'
}

def __col(c): return __color[c]
__write = _sys.stdout.write

__log_print = evalcore.log_print
__die = evalcore.die


def parse_parameters(argv):
    ''' Parses shell parameters
    Returns tuple (params, options)
    `Params` are global or one-time parameters
    `options` are problem-specific parameters
    '''
    from optparse import OptionParser
    "Parses parameters; uses `defaults` and saves to dict `options`"
    parser = OptionParser(usage="%prog executabil [options]")
    rules = [
        (['-p', '--prob'], {'metavar':'...', 'dest':'path', 'help':'Numele problemei (si directorului de teste)'}),
        (['-n', '--name'], {'dest':'nume', 'help':'numele problemei in teste, daca diferit de -p'}),
        (['-d'],           {'dest':'debug', 'action':"store_true", 'default':False, 'help':'debug pentru testele picate'}),
        (['-t', '--time'], {'type':'int', 'help':'time limit in milisecunde'}),
        (['-m', '--memory'], {'metavar':'MEM', 'type':"int", 'help':'limita memorie, kb'}),
        (['-q', '--quiet'], {'dest':'quiet', 'action':'store_true', 'default':False, 'help':'Nu afiseaza decat scorul'}),
        (['-O'], {'dest':'output', 'help':'Fisier pentru salvarea borderoului'}),
        (['--chroot'],     {'action':'store_true', 'default':False,
                            'help':'''Foloseste chroot (jrun are setuid/esti root)
                            Comanda pt jrun: %s
                            ''' % evalcore.eval_options['chroot_command']}),
        (['-r', '--repo'], {'action': 'store_true', 'default':False, 'help':'intotdeauna foloseste repository'}),
        (['-i', '--inf'],  {'help':'''Format fisier intrare (default %default)
                                   %s se expandeaza ca numele problemei, ? ca nr testului'''}),
        (['-o', '--outf'], {'help':'Format fisier iesire'}),
        (['--verif'],      {'help':'''Program verificator
                                   Parametri: VERIF fisier.in fisier.out fisier.ok
                                   Returneaza: "mesaj\\n(procent punctaj din max)"
                                   Exemplu: Gresit\\n0 ; sau: OK\\n100'''})
    ]
    for i, j in rules: parser.add_option(*i, **j)
    parser.set_defaults(**evalcore.defaults)
    (opts, args) = parser.parse_args(argv)
    if len(args) < 2:
        __die("Trebuie sa precizezi numele executabilului pe care-l evaluez")
    if len(args) > 2:
        __die("Parametru in plus '%s'" % args[2])


    params_list = ('debug', 'repo', 'quiet', 'output', 'chroot')
    params = dict((i, opts.__dict__[i]) for i in params_list)
    options_list = ('inf', 'outf', 'nume', 'path', 'memory', 'time', 'verif')
    options = dict((i, opts.__dict__[i]) for i in options_list)

    params['executabil'] = _re.sub('^./', '', args[1])

    return (params, options)


__headers = {
    'id': 'Test',
    'group_score': 'Group',
    'real_score': 'Score',
    'time': 'Time',
    'memory': 'Memory',
    'verdict': 'Verdict'
}
__lens = {
    'id': 6,
    'group_score': 5,
    'real_score': 5,
    'time': 10,
    'memory': 10,
    'verdict': 13,
}

def __adapt(str):
    if str == 'Time limit exceeded.':
        return 'TLE'
    elif str == 'Wall time limit exceeded.':
        return 'WTLE'
    elif str == 'Memory limit exceeded.':
        return 'MLE'
    return str


def print_results(problema, executabil, params = {}, **kwargs):
    ''' Main show() function for the console interface
        **kwargs -> only among: ['debug' (Bool), 'chroot' (Bool)]
    '''
    # HACK to get chroot and debug parameters to evaluate()
    for i in ('chroot', 'debug'):
        if i not in params: continue
        kwargs[i] = params[i]
    tester = problema.evaluate(executabil, **kwargs)
    # Display table if we have an output to print to
    display_table = hasattr(evalcore, 'output') and evalcore.output

    score = real_score = 0
    nr_teste = 0
    group_score = {}

    visual_notice =__name__ == '__main__' and \
        params.get('output') not in [None, '-'] or params.get('quiet')

    having_groups = problema.groupof
    having_real_scores = problema.punctaje
    columns = ['id'] + \
        (having_real_scores and ['real_score'] or []) + \
        (having_groups and ['group_score'] or []) + \
        ['time', 'memory', 'verdict']

    if having_groups and not having_real_scores:
        __die('For now, you must provide tests.txt (real scores) in order to use groups')

    # evaluate() will automatically raise an exception if there are no tests

    for nr, (test, progress_pair) in enumerate(tester):
        if visual_notice:
            _sys.stderr.write('%d ' % test['id'])
            _sys.stderr.flush()
        # intotdeauna trebuie sa fie ori una ori alta
        # FIXME: daca e grup, nu adaug pur si simplu. bag in group_score
        crt_score = (having_real_scores and (test['real_score'],) or (test['score'],))[0]

        content = {}
        # HANDLING GROUPS
        # ---------------
        if having_groups:
            group = test['group']
            if group is None:
                # FIXME: daca nu exista real_score?
                # poate trebuie sa explicitez score, sa nu mai existe decat real_score?
                content['group_score'] = test['real_score']
                real_score += content['group_score']
            else:
                gr_l, gr_r = test['group_range']
                tmp_score = group_score.get(group, 0)
                if test['highlight'] != 'green':
                    tmp_score = False
                if tmp_score is not False:
                    # FIXME same here
                    tmp_score += test['real_score']
                group_score[group] = tmp_score
                if gr_r > test['id']:
                    content['group_score'] = '~'
                else:
                    content['group_score'] = group_score[group] or 0
                    real_score += content['group_score']
        else:
            if having_real_scores:
                real_score += test['real_score']
            else:
                score += test['score']

        nr_teste = nr + 1

        if display_table:
            for i in ('id', 'real_score', 'time', 'memory'):
                content[i] = test.get(i)

            content['verdict'] = test['verdict'] == 'FAIL' and \
                __adapt(test['jrun_message']) or test['verdict']

            # Baga cap de tabel, daca suntem la primul test
            if nr == 0:
                line = [__headers[i].ljust(__lens[i]) for i in columns]
                __log_print(' '.join(line), 1)
                line = ['-'*__lens[i] for i in columns]
                __log_print(' '.join(line), 1)

            line = [str(content[i]).ljust(__lens[i]) for i in columns]
            # Colorization. still a tad Hacky!
            if evalcore.output.isatty():
                line[-1] = __col(test['highlight']) + line[-1] + __col('normal')
                if having_real_scores:
                    line[1] = (test['highlight']=='green' and __col('maialb') or '') + str(line[1]) + __col('normal')
            __log_print(' '.join(line), 1)

            # OLD VERSION
            # Will be gone in a few weeks I guess
            '''
            lenz = [12, 10, 10, 10, 10, 10]
            order = ['id', 'real_score', 'time', 'memory', 'verdict'] # 'jrun_message'

            line = map(test.get, order)
            line[4] = test['verdict'] == 'FAIL' and adapt(test['jrun_message']) or line[4]
            line[0] = 'Testul %s' % line[0]

            # removal of empty fields, and making everything string
            line = [str(i) for i in line if i is not None]

            # give it table structure
            line = [j.ljust(lenz[i]) for i, j in enumerate(line)]
            # Colorization
            if evalcore.output.isatty():
                line[-1] = col(test['highlight']) + line[-1] + col('normal')
                if 'real_score' in test:
                    line[1] = (test['highlight']=='green' and col('maialb') or '') + str(line[1]) + col('normal')
            __log_print(''.join(line), 1)
            '''
            if 'debug_data' in test:
                path1 = problema.opts['nume'] + '-debug'
                path2 = '%d.diff' % test['id']
                try:
                    if not _os.path.isdir(path1):
                        _os.mkdir(path1)
                    open(join(path1, path2), 'wt').write(''.join(test['debug_data']))
                except (OSError, IOError), e:
                    print >>_sys.stderr, ("Couldn't write debug data: " + str(e))

    if visual_notice and _sys.stdout.isatty():
        print >>_sys.stderr

    puncte = real_score or int(round(score/nr_teste))

    # Table "ending"
    __log_print(' '.join('-'*__lens[i] for i in columns), 1)

    # Afisarea scorului tot in tabel !
    content = {'id': 'Total'}
    if having_real_scores:
        if having_groups:
            content['group_score'] = puncte
        else:
            content['real_score'] = puncte
        line = [str(content.get(i, '')).ljust(__lens[i]) for i in columns]
        __log_print(' '.join(line), 1)
    else:
        __log_print('Total: %d' % puncte, 1)


    # FIXME: a little dubious. maybe do it IF visual_notice?
    if not display_table or params.get('output') not in [None, '-']:
        print puncte


def main(argv=None):
    'Parse parameters (argv or _sys.argv) and evaluate one single executable'
    if argv is None:
        argv = _sys.argv
    params, options = parse_parameters(argv)
    executabil = params['executabil']
    # Daca este specificat doar executabilul
    if not options.get('path') and not options.get('nume'):
        options['nume'] = _os.path.basename(executabil)
    # Shortcut. Don't initialize object if it's just a one-time failed run
    # Problema.evaluate() makes this check too
    if not isfile(executabil):
        __die('The executable to be evaluated does not exist')

    evalcore.setup_output(quiet = params['quiet'], output_path = params['output'], verbosity = 2)

    problema = evalcore.Problema(params = params, configure = True, **options)
    print_results(problema, executabil, params)


if __name__ == "__main__":
    #from time import clock as T
    try:
        #t = T()
        main()
        #print >>_sys.stderr, 'Total time spent in python code: ', T() - t
        _sys.exit(0)
    except evalcore.EvalError, e:
        print >>_sys.stderr, e
    except IOError, e:
        print >>_sys.stderr, e
    _sys.exit(1)
